package ch.epfl.ppo.collections;

import java.util.Arrays;
import java.util.Iterator;
import java.util.NoSuchElementException;

public final class ArrayList<E> implements List<E> {
    private int size = 0;
    private Object[] array = new Object[8];

    @Override
    public boolean isEmpty() {
        return size == 0;
    }

    @Override
    public int size() {
        return size;
    }

    @Override
    public void add(E newElem) {
        if (size == array.length)
            array = Arrays.copyOf(array, array.length * 2);
        array[size] = newElem;
        size += 1;
    }

    @Override
    public void remove(int i) {
        checkIndex(i);
        System.arraycopy(array, i + 1, array, i, size - 1 - i);
        array[size - 1] = null;
        size -= 1;
    }

    @SuppressWarnings("unchecked")
    @Override
    public E get(int i) {
        checkIndex(i);
        return (E)array[i];
    }

    @Override
    public void set(int i, E newElem) {
        // TODO Auto-generated method stub

    }

    @Override
    public Iterator<E> iterator() {
        return new ALIterator<>(this);
    }

    private void checkIndex(int i) {
        if (! (0 <= i && i < size))
            throw new IndexOutOfBoundsException("index out of bounds: " + i);
    }

    private static final class ALIterator<E> implements Iterator<E> {
        private final ArrayList<E> list;
        private int index;

        public ALIterator(ArrayList<E> list) {
            this.list = list;
            this.index = 0;
        }

        @Override
        public boolean hasNext() {
            return index < list.size();
        }

        @Override
        public E next() {
            if (! hasNext())
                throw new NoSuchElementException();
            return list.get(index++);
        }

        @Override
        public void remove() {
            list.remove(index - 1);
        }
    }
}
